searchData={"content_type":"text/markdown","items":[{"doc":"Extend `Commanded` package. A swiss army knife for applications following Domain-Driven Design (DDD), Event Sourcing\n(ES), and Command and Query Responsibility Segregation (CQRS).","ref":"OnePiece.Commanded.html","title":"OnePiece.Commanded","type":"module"},{"doc":"","ref":"OnePiece.Commanded.html#cast_to/3","title":"OnePiece.Commanded.cast_to/3","type":"function"},{"doc":"","ref":"OnePiece.Commanded.html#ignore_error/2","title":"OnePiece.Commanded.ignore_error/2","type":"function"},{"doc":"","ref":"OnePiece.Commanded.html#increase_failure_counter/1","title":"OnePiece.Commanded.increase_failure_counter/1","type":"function"},{"doc":"","ref":"OnePiece.Commanded.html#skip_or_retry/2","title":"OnePiece.Commanded.skip_or_retry/2","type":"function"},{"doc":"","ref":"OnePiece.Commanded.html#skip_or_retry/3","title":"OnePiece.Commanded.skip_or_retry/3","type":"function"},{"doc":"","ref":"OnePiece.Commanded.html#tracing_from_metadata/1","title":"OnePiece.Commanded.tracing_from_metadata/1","type":"function"},{"doc":"","ref":"OnePiece.Commanded.html#tracing_from_metadata/2","title":"OnePiece.Commanded.tracing_from_metadata/2","type":"function"},{"doc":"Stops the aggregate after a command, event or error.","ref":"OnePiece.Commanded.Aggregate.StatelessLifespan.html","title":"OnePiece.Commanded.Aggregate.StatelessLifespan","type":"module"},{"doc":"Stops the aggregate after a command.\n    iex> OnePiece.Commanded.Aggregate.StatelessLifespan.after_command(%MyCommandOne{})\n    :stop","ref":"OnePiece.Commanded.Aggregate.StatelessLifespan.html#after_command/1","title":"OnePiece.Commanded.Aggregate.StatelessLifespan.after_command/1","type":"function"},{"doc":"Stops the aggregate after an error.\n    iex> OnePiece.Commanded.Aggregate.StatelessLifespan.after_error({:error, :something_happened})\n    :stop","ref":"OnePiece.Commanded.Aggregate.StatelessLifespan.html#after_error/1","title":"OnePiece.Commanded.Aggregate.StatelessLifespan.after_error/1","type":"function"},{"doc":"Stops the aggregate after an event.\n    iex> OnePiece.Commanded.Aggregate.StatelessLifespan.after_event(%DepositAccountOpened{})\n    :stop","ref":"OnePiece.Commanded.Aggregate.StatelessLifespan.html#after_event/1","title":"OnePiece.Commanded.Aggregate.StatelessLifespan.after_event/1","type":"function"},{"doc":"Defines a module as a \"Command Handler\". For more information about commands, please read the following:\n\n- [CQRS pattern](https://docs.microsoft.com/en-us/azure/architecture/patterns/cqrs)","ref":"OnePiece.Commanded.CommandHandler.html","title":"OnePiece.Commanded.CommandHandler","type":"module"},{"doc":"Convert the module into a `Commanded.Commands.Handler`.\n\nUse `Commanded.Aggregate.Multi` to generate multiple events from\na single command. This can be useful when you want to emit multiple events\nthat depend upon the aggregate state being updated.","ref":"OnePiece.Commanded.CommandHandler.html#__using__/1","title":"OnePiece.Commanded.CommandHandler.__using__/1","type":"macro"},{"doc":"defmodule MyCommandHandler do\n      use OnePiece.Commanded.CommandHandler\n\n      @impl Commanded.Commands.Handler\n      def handle(account, command) do\n        account\n        |> Multi.new()\n        |> Multi.execute(&withdraw_money(&1, command.amount))\n        |> Multi.execute(&check_balance/1)\n      end\n\n      defp withdraw_money(account, amount) do\n        # ...\n      end\n\n      defp check_balance(account, amount) do\n        # ...\n      end\n    end","ref":"OnePiece.Commanded.CommandHandler.html#__using__/1-usage","title":"Usage - OnePiece.Commanded.CommandHandler.__using__/1","type":"macro"},{"doc":"A JSONB serializer based on events defined by `OnePiece.Commanded.Event`.\n\nIt uses `OnePiece.Commanded.Event` to cast the event to the correct type, by proxying, using `Ecto.Schema`s and\n`Ecto.Changeset`s.","ref":"OnePiece.Commanded.EventStore.JsonbSerializer.html","title":"OnePiece.Commanded.EventStore.JsonbSerializer","type":"module"},{"doc":"To use this serializer, add it to your `config.exs`:\n\n    config :my_app, MyApp.EventStore,\n      serializer: OnePiece.Commanded.EventStore.JsonbSerializer,\n      types: EventStore.PostgresTypes","ref":"OnePiece.Commanded.EventStore.JsonbSerializer.html#module-configuring","title":"Configuring - OnePiece.Commanded.EventStore.JsonbSerializer","type":"module"},{"doc":"Deserialize given JSON binary data to the expected type.\n\nIt is already a map since `EventStore.PostgresTypes` will take care of the deserialization. Then, it will use\n`OnePiece.Commanded.Event` to cast the event to the correct type.","ref":"OnePiece.Commanded.EventStore.JsonbSerializer.html#deserialize/2","title":"OnePiece.Commanded.EventStore.JsonbSerializer.deserialize/2","type":"function"},{"doc":"Serialize given term to JSON binary data.\n\nIt is just a passthrough, since `EventStore.PostgresTypes` will take care of the serialization.","ref":"OnePiece.Commanded.EventStore.JsonbSerializer.html#serialize/1","title":"OnePiece.Commanded.EventStore.JsonbSerializer.serialize/1","type":"function"},{"doc":"A Swiss Army Knife Helper Module.","ref":"OnePiece.Commanded.Helpers.html","title":"OnePiece.Commanded.Helpers","type":"module"},{"doc":"Copy the information from the `source` map into the given `target` map.\n\n    iex> OnePiece.Commanded.Helpers.cast_to(%{}, %{name: \"ubi-wan\", last_name: \"kenobi\"}, [:last_name])\n    %{last_name: \"kenobi\"}","ref":"OnePiece.Commanded.Helpers.html#cast_to/3","title":"OnePiece.Commanded.Helpers.cast_to/3","type":"function"},{"doc":"Deprecated, it has the same behavior as `OnePiece.Commanded.Id.new/0`.","ref":"OnePiece.Commanded.Helpers.html#generate_uuid/0","title":"OnePiece.Commanded.Helpers.generate_uuid/0","type":"function"},{"doc":"Ignores a specific error from a command dispatch result.\n\nThis function takes a dispatch result and an error term. If the result represents an error that matches the provided\nerror term, the function returns `:ok`. Otherwise, it returns the original result.\n\nThis is useful when an error condition should be treated as a successful operation under specific circumstances.","ref":"OnePiece.Commanded.Helpers.html#ignore_error/2","title":"OnePiece.Commanded.Helpers.ignore_error/2","type":"function"},{"doc":"iex> OnePiece.Commanded.Helpers.ignore_error({:error, :idempotency_failure}, :idempotency_failure)\n    :ok\n\n    iex> OnePiece.Commanded.Helpers.ignore_error({:error, :something_went_wrong}, :idempotency_failure)\n    {:error, :something_went_wrong}\n\n    iex> OnePiece.Commanded.Helpers.ignore_error(:ok, :idempotency_failure)\n    :ok\n\n    iex> OnePiece.Commanded.Helpers.ignore_error({:ok, %{name: \"Billy\"}}, :idempotency_failure)\n    {:ok, %{name: \"Billy\"}}","ref":"OnePiece.Commanded.Helpers.html#ignore_error/2-examples","title":"Examples - OnePiece.Commanded.Helpers.ignore_error/2","type":"function"},{"doc":"Increase the failure counter from `t:Commanded.Event.FailureContext.t/0` context by one.\n\n      iex> OnePiece.Commanded.Helpers.increase_failure_counter(%Commanded.Event.FailureContext{context: %{failures_count: 1}})\n      %{failures_count: 2}","ref":"OnePiece.Commanded.Helpers.html#increase_failure_counter/1","title":"OnePiece.Commanded.Helpers.increase_failure_counter/1","type":"function"},{"doc":"Returns `skip` or a `retry` response.\n\nWhen the `c:Commanded.Application.dispatch/1` or `c:Commanded.Application.dispatch/2` returns an `:skip` otherwise,\nreturns a `:retry` response. Useful when you are doing error handling in your `c:Commanded.Event.Handler.error/3`.\n\n    iex> success_dispatch = fn _ -> :ok end\n    ...> OnePiece.Commanded.Helpers.skip_or_retry(success_dispatch.(%{}), %{})\n    :skip\n\n    iex> success_dispatch = fn _ -> {:ok, %{}} end\n    ...> OnePiece.Commanded.Helpers.skip_or_retry(success_dispatch.(%{}), %{})\n    :skip\n\n    iex> failure_dispatch = fn _ -> {:error, :ooops} end\n    ...> OnePiece.Commanded.Helpers.skip_or_retry(failure_dispatch.(%{}), %{failures: 1})\n    {:retry, %{failures: 1}}","ref":"OnePiece.Commanded.Helpers.html#skip_or_retry/2","title":"OnePiece.Commanded.Helpers.skip_or_retry/2","type":"function"},{"doc":"Returns `skip` or a `retry` response with a given delay.\n\nWhen the `c:Commanded.Application.dispatch/1` or `c:Commanded.Application.dispatch/2` returns an `:skip` otherwise,\nreturns a `:retry` response. Useful when you are doing error handling in your `c:Commanded.Event.Handler.error/3`.\n\n    iex> success_dispatch = fn _ -> :ok end\n    ...> OnePiece.Commanded.Helpers.skip_or_retry(success_dispatch.(%{}), 5_000, %{})\n    :skip\n\n    iex> success_dispatch = fn _ -> {:ok, %{}} end\n    ...> OnePiece.Commanded.Helpers.skip_or_retry(success_dispatch.(%{}), 5_000, %{})\n    :skip\n\n    iex> failure_dispatch = fn _ -> {:error, :ooops} end\n    ...> OnePiece.Commanded.Helpers.skip_or_retry(failure_dispatch.(%{}), 5_000, %{failures: 1})\n    {:retry, 5_000, %{failures: 1}}","ref":"OnePiece.Commanded.Helpers.html#skip_or_retry/3","title":"OnePiece.Commanded.Helpers.skip_or_retry/3","type":"function"},{"doc":"Transforms the given `source` map or struct into the `target` struct.","ref":"OnePiece.Commanded.Helpers.html#struct_from/2","title":"OnePiece.Commanded.Helpers.struct_from/2","type":"function"},{"doc":"Returns a keyword list containing the \"correlation id\" and \"causation id\" tracing.\n\n    iex> OnePiece.Commanded.Helpers.tracing_from_metadata(%{\n    ...>   event_id: \"26eb06fe-9ba6-4f58-a2dd-2bdba73de4f2\",\n    ...>   correlation_id: \"f634ba94-145c-4fa7-bf7f-0d73dd83b446\"\n    ...> })\n    ...>\n    [causation_id: \"26eb06fe-9ba6-4f58-a2dd-2bdba73de4f2\", correlation_id: \"f634ba94-145c-4fa7-bf7f-0d73dd83b446\"]\n\nUseful when dispatching commands to copy-forward `t:Commanded.Event.Handler.metadata/0` tracing information.\n\n    defmodule MyProcessor do\n        application: MyApp,\n      use Commanded.Event.Handler,\n        name: \"my_processor\"\n\n      alias OnePiece.Commanded.Helpers\n\n      def handle(%MyEvent{} = event, metadata) do\n        MyApp.dispatch(\n          %MyCommand{},\n          # copy-forward the information\n          Helpers.tracing_from_metadata(metadata)\n        )\n      end\n    end","ref":"OnePiece.Commanded.Helpers.html#tracing_from_metadata/1","title":"OnePiece.Commanded.Helpers.tracing_from_metadata/1","type":"function"},{"doc":"Adds the \"correlation id\" and \"causation id\" tracing to an existing keyword list configuration option.\n\n    iex> OnePiece.Commanded.Helpers.tracing_from_metadata([timeout: 30_000], %{\n    ...>   event_id: \"26eb06fe-9ba6-4f58-a2dd-2bdba73de4f2\",\n    ...>   correlation_id: \"f634ba94-145c-4fa7-bf7f-0d73dd83b446\"\n    ...> })\n    ...>\n    [timeout: 30_000, causation_id: \"26eb06fe-9ba6-4f58-a2dd-2bdba73de4f2\", correlation_id: \"f634ba94-145c-4fa7-bf7f-0d73dd83b446\"]\n\nUseful when dispatching commands to copy-forward the `t:Commanded.Event.Handler.metadata/0` tracing information and\nwants to also add other keyword list options.\n\n    defmodule MyProcessor do\n      use Commanded.Event.Handler,\n        application: MyApp,\n        name: \"my_processor\"\n\n      alias OnePiece.Commanded.Helpers\n\n      def handle(%MyEvent{} = event, metadata) do\n        MyApp.dispatch(\n          %MyCommand{},\n          # copy-forward the information\n          Helpers.tracing_from_metadata([timeout: 30_000], metadata)\n        )\n      end\n    end","ref":"OnePiece.Commanded.Helpers.html#tracing_from_metadata/2","title":"OnePiece.Commanded.Helpers.tracing_from_metadata/2","type":"function"},{"doc":"","ref":"OnePiece.Commanded.Helpers.html#t:commanded_dispatch_response/0","title":"OnePiece.Commanded.Helpers.commanded_dispatch_response/0","type":"type"},{"doc":"","ref":"OnePiece.Commanded.Helpers.html#t:error_context/0","title":"OnePiece.Commanded.Helpers.error_context/0","type":"type"},{"doc":"A module for dealing IDs.","ref":"OnePiece.Commanded.Id.html","title":"OnePiece.Commanded.Id","type":"behaviour"},{"doc":"Generates a UUID using the version 4 scheme, as described in\n[RFC 4122](https://datatracker.ietf.org/doc/html/rfc4122).\n\n    iex> OnePiece.Commanded.Id.new() |> String.length()\n    36","ref":"OnePiece.Commanded.Id.html#new/0","title":"OnePiece.Commanded.Id.new/0","type":"function"},{"doc":"Generates a new ID.","ref":"OnePiece.Commanded.Id.html#c:new/0","title":"OnePiece.Commanded.Id.new/0","type":"callback"},{"doc":"Defines a module as a \"Query Handler\". For more information about queries, please read the following:\n\n- [CQRS pattern](https://docs.microsoft.com/en-us/azure/architecture/patterns/cqrs)","ref":"OnePiece.Commanded.QueryHandler.html","title":"OnePiece.Commanded.QueryHandler","type":"behaviour"},{"doc":"Convert the module into a `OnePiece.Commanded.QueryHandler`.","ref":"OnePiece.Commanded.QueryHandler.html#__using__/1","title":"OnePiece.Commanded.QueryHandler.__using__/1","type":"macro"},{"doc":"defmodule MyQueryHandler do\n      use OnePiece.Commanded.QueryHandler\n      import Ecto.Query, only: [from: 2]\n\n      @impl OnePiece.Commanded.QueryHandler\n      def handle(params) do\n        query = from u in User,\n                  where: u.age > 18 or is_nil(params.email),\n                  select: u\n\n        {:ok, Repo.all(query)}\n      end\n    end","ref":"OnePiece.Commanded.QueryHandler.html#__using__/1-usage","title":"Usage - OnePiece.Commanded.QueryHandler.__using__/1","type":"macro"},{"doc":"Handle the incoming `params` and return the result data.","ref":"OnePiece.Commanded.QueryHandler.html#c:handle/1","title":"OnePiece.Commanded.QueryHandler.handle/1","type":"callback"},{"doc":"","ref":"OnePiece.Commanded.QueryHandler.html#t:error/0","title":"OnePiece.Commanded.QueryHandler.error/0","type":"type"},{"doc":"","ref":"OnePiece.Commanded.QueryHandler.html#t:view_model/0","title":"OnePiece.Commanded.QueryHandler.view_model/0","type":"type"},{"doc":"Implements `Commanded.EventStore.TypeProvider` behavior. Using macros to generate the behavior.","ref":"OnePiece.Commanded.TypeProvider.html","title":"OnePiece.Commanded.TypeProvider","type":"module"},{"doc":"Imports all the types from another module defined by `OnePiece.Commanded.TypeProvider`.","ref":"OnePiece.Commanded.TypeProvider.html#import_type_provider/1","title":"OnePiece.Commanded.TypeProvider.import_type_provider/1","type":"macro"},{"doc":"defmodule UserTypeProvider do\n      use OnePiece.Commanded.TypeProvider\n      # ...\n    end\n\n    defmodule MyAppTypeProvider do\n      use OnePiece.Commanded.TypeProvider\n\n      import_type_provider UserTypeProvider\n    end","ref":"OnePiece.Commanded.TypeProvider.html#import_type_provider/1-example","title":"Example - OnePiece.Commanded.TypeProvider.import_type_provider/1","type":"macro"},{"doc":"Registers a mapping from an name string to an Elixir Module that defines a struct.","ref":"OnePiece.Commanded.TypeProvider.html#register_type/2","title":"OnePiece.Commanded.TypeProvider.register_type/2","type":"macro"},{"doc":"defmodule MyTypeProvider do\n      use OnePiece.Commanded.TypeProvider,\n        prefix: \"accounts.\" # optional, adds the prefix to the type name\n\n      register_type \"account_created\", AccountCreated\n    end","ref":"OnePiece.Commanded.TypeProvider.html#register_type/2-example","title":"Example - OnePiece.Commanded.TypeProvider.register_type/2","type":"macro"},{"doc":"Defines \"Aggregate\" modules.","ref":"OnePiece.Commanded.Aggregate.html","title":"OnePiece.Commanded.Aggregate","type":"behaviour"},{"doc":"Convert the module into a `Aggregate` behaviour and a `t:t/0`.\n\nIt adds an `apply/2` callback to the module as a fallback, return the aggregate as it is.","ref":"OnePiece.Commanded.Aggregate.html#__using__/1","title":"OnePiece.Commanded.Aggregate.__using__/1","type":"macro"},{"doc":"- `OnePiece.Commanded.Entity`","ref":"OnePiece.Commanded.Aggregate.html#__using__/1-using","title":"Using - OnePiece.Commanded.Aggregate.__using__/1","type":"macro"},{"doc":"defmodule Account do\n      use OnePiece.Commanded.Aggregate, identifier: :name\n\n      embedded_schema do\n        field :description, :string\n      end\n\n      @impl OnePiece.Commanded.Aggregate\n      def apply(%Account{} = aggregate, %AccountOpened{} = event) do\n        aggregate\n        |> Map.put(:name, event.name)\n        |> Map.put(:description, event.description)\n      end\n    end","ref":"OnePiece.Commanded.Aggregate.html#__using__/1-usage","title":"Usage - OnePiece.Commanded.Aggregate.__using__/1","type":"macro"},{"doc":"Apply a given event to the aggregate returning the new aggregate state.","ref":"OnePiece.Commanded.Aggregate.html#c:apply/2","title":"OnePiece.Commanded.Aggregate.apply/2","type":"callback"},{"doc":"def apply(%MyAggregate{} = aggregate, %MyEvent{} = event) do\n      aggregate\n      |> Map.put(:name, event.name)\n      |> Map.put(:description, event.description)\n    end","ref":"OnePiece.Commanded.Aggregate.html#c:apply/2-example","title":"Example - OnePiece.Commanded.Aggregate.apply/2","type":"callback"},{"doc":"","ref":"OnePiece.Commanded.Aggregate.html#t:event/0","title":"OnePiece.Commanded.Aggregate.event/0","type":"type"},{"doc":"A struct that represents an aggregate.","ref":"OnePiece.Commanded.Aggregate.html#t:t/0","title":"OnePiece.Commanded.Aggregate.t/0","type":"type"},{"doc":"Defines \"Command\" modules.","ref":"OnePiece.Commanded.Command.html","title":"OnePiece.Commanded.Command","type":"module"},{"doc":"Converts the module into an `t:t/0`.","ref":"OnePiece.Commanded.Command.html#__using__/1","title":"OnePiece.Commanded.Command.__using__/1","type":"macro"},{"doc":"- `OnePiece.Commanded.ValueObject`","ref":"OnePiece.Commanded.Command.html#__using__/1-using","title":"Using - OnePiece.Commanded.Command.__using__/1","type":"macro"},{"doc":"defmodule MyCommand do\n      use OnePiece.Commanded.Command, aggregate_identifier: :id\n\n      embedded_schema do\n        # ...\n      end\n    end\n\nYou can also define a custom type as the aggregate identifier:\n\n    defmodule IdentityRoleId do\n      use OnePiece.Commanded.ValueObject\n\n      embedded_schema do\n        field :identity_id, :string\n        field :role_id, :string\n      end\n    end\n\n    defmodule AssignRole do\n      use OnePiece.Commanded.Command,\n        aggregate_identifier: {:id, IdentityRoleId}\n\n      embedded_schema do\n        # ...\n      end\n    end","ref":"OnePiece.Commanded.Command.html#__using__/1-usage","title":"Usage - OnePiece.Commanded.Command.__using__/1","type":"macro"},{"doc":"The aggregate identifier key of the event.\n\nIf it's a tuple, the type must be a module that implements the `OnePiece.Commanded.ValueObject` module or [`Ecto` built-in types](https://hexdocs.pm/ecto/Ecto.Schema.html#module-types-and-casting)","ref":"OnePiece.Commanded.Command.html#t:aggregate_identifier_opt/0","title":"OnePiece.Commanded.Command.aggregate_identifier_opt/0","type":"type"},{"doc":"A struct that represents a command.","ref":"OnePiece.Commanded.Command.html#t:t/0","title":"OnePiece.Commanded.Command.t/0","type":"type"},{"doc":"Defines \"Entity\" modules.","ref":"OnePiece.Commanded.Entity.html","title":"OnePiece.Commanded.Entity","type":"module"},{"doc":"Converts the module into an `t:t/0`.","ref":"OnePiece.Commanded.Entity.html#__using__/1","title":"OnePiece.Commanded.Entity.__using__/1","type":"macro"},{"doc":"- `OnePiece.Commanded.ValueObject`","ref":"OnePiece.Commanded.Entity.html#__using__/1-using","title":"Using - OnePiece.Commanded.Entity.__using__/1","type":"macro"},{"doc":"defmodule MyEntity do\n      use OnePiece.Commanded.Entity, identifier: :id\n\n      embedded_schema do\n        # ...\n      end\n    end\n\nYou can also define a custom type as the identifier:\n\n    defmodule IdentityRoleId do\n      use OnePiece.Commanded.ValueObject\n\n      embedded_schema do\n        field :identity_id, :string\n        field :role_id, :string\n      end\n    end\n\n    defmodule IdentityRole do\n      use OnePiece.Commanded.Entity,\n        identifier: {:id, IdentityRoleId}\n\n      embedded_schema do\n        # ...\n      end\n    end","ref":"OnePiece.Commanded.Entity.html#__using__/1-usage","title":"Usage - OnePiece.Commanded.Entity.__using__/1","type":"macro"},{"doc":"The identity key of the entity.\n\nIf it's a tuple, the type must be a module that implements the `OnePiece.Commanded.ValueObject` module or [`Ecto` built-in types](https://hexdocs.pm/ecto/Ecto.Schema.html#module-types-and-casting)","ref":"OnePiece.Commanded.Entity.html#t:identifier_opt/0","title":"OnePiece.Commanded.Entity.identifier_opt/0","type":"type"},{"doc":"The identity of an entity.","ref":"OnePiece.Commanded.Entity.html#t:identity/0","title":"OnePiece.Commanded.Entity.identity/0","type":"type"},{"doc":"A struct that represents an entity.","ref":"OnePiece.Commanded.Entity.html#t:t/0","title":"OnePiece.Commanded.Entity.t/0","type":"type"},{"doc":"Defines \"Event\" modules.","ref":"OnePiece.Commanded.Event.html","title":"OnePiece.Commanded.Event","type":"module"},{"doc":"Converts the module into an `t:t/0`.","ref":"OnePiece.Commanded.Event.html#__using__/1","title":"OnePiece.Commanded.Event.__using__/1","type":"macro"},{"doc":"- `OnePiece.Commanded.ValueObject`","ref":"OnePiece.Commanded.Event.html#__using__/1-using","title":"Using - OnePiece.Commanded.Event.__using__/1","type":"macro"},{"doc":"defmodule MyEvent do\n      use OnePiece.Commanded.Event, aggregate_identifier: :id\n\n      embedded_schema do\n        # ...\n      end\n    end\n\nYou can also define a custom type as the aggregate identifier:\n\n    defmodule IdentityRoleId do\n      use OnePiece.Commanded.ValueObject\n\n      embedded_schema do\n        field :identity_id, :string\n        field :role_id, :string\n      end\n    end\n\n    defmodule IdentityRoleAssigned do\n      use OnePiece.Commanded.Event,\n        aggregate_identifier: {:id, IdentityRoleId}\n\n      embedded_schema do\n        # ...\n      end\n    end","ref":"OnePiece.Commanded.Event.html#__using__/1-usage","title":"Usage - OnePiece.Commanded.Event.__using__/1","type":"macro"},{"doc":"The aggregate identifier key of the event.\n\nIf it's a tuple, the type must be a module that implements the `OnePiece.Commanded.ValueObject` module or [`Ecto` built-in types](https://hexdocs.pm/ecto/Ecto.Schema.html#module-types-and-casting)","ref":"OnePiece.Commanded.Event.html#t:aggregate_identifier_opt/0","title":"OnePiece.Commanded.Event.aggregate_identifier_opt/0","type":"type"},{"doc":"A struct that represents an event.","ref":"OnePiece.Commanded.Event.html#t:t/0","title":"OnePiece.Commanded.Event.t/0","type":"type"},{"doc":"Defines \"Value Object\" modules.","ref":"OnePiece.Commanded.ValueObject.html","title":"OnePiece.Commanded.ValueObject","type":"module"},{"doc":"Converts the module into an `Ecto.Schema` and add factory functions to create structs.","ref":"OnePiece.Commanded.ValueObject.html#__using__/1","title":"OnePiece.Commanded.ValueObject.__using__/1","type":"macro"},{"doc":"- `Ecto.Schema`\n- `Ecto.Type`","ref":"OnePiece.Commanded.ValueObject.html#__using__/1-using","title":"Using - OnePiece.Commanded.ValueObject.__using__/1","type":"macro"},{"doc":"- `Jason.Encoder`","ref":"OnePiece.Commanded.ValueObject.html#__using__/1-derives","title":"Derives - OnePiece.Commanded.ValueObject.__using__/1","type":"macro"},{"doc":"defmodule MyValueObject do\n      use OnePiece.Commanded.ValueObject\n\n      embedded_schema do\n        field :title, :string\n        # ...\n      end\n    end\n\n    {:ok, my_value} = MyValueObject.new(%{title: \"Hello, World!\"})","ref":"OnePiece.Commanded.ValueObject.html#__using__/1-usage","title":"Usage - OnePiece.Commanded.ValueObject.__using__/1","type":"macro"},{"doc":"This module helps with test cases for testing aggregate states, and command handlers.","ref":"OnePiece.Commanded.TestSupport.CommandHandlerCase.html","title":"OnePiece.Commanded.TestSupport.CommandHandlerCase","type":"module"},{"doc":"After import the test support file, you should be able to have the module in your test files.\n\n    defmodule MyAggregateTest do\n      use OnePiece.Commanded.TestSupport.CommandHandlerCase,\n        aggregate: MyAggregate,\n        # You can also pass `handler` if you are using Command Handler modules\n        # handler: MyCommandHandler,\n        async: true\n\n      describe \"my aggregate\" do\n        test \"should do something\" do\n          assert_events(\n            [%InitialEvent{}],\n            %DoSomething{},\n            [%SomethingHappened{}]\n          )\n        end\n\n        test \"the state\" do\n          assert_state(\n            [%InitialEvent{}],\n            %DoSomething{},\n            %MyAggregate{}\n          )\n        end\n\n        test \"the error\" do\n          assert_error(\n            [%InitialEvent{}],\n            %DoSomething{},\n            :already_exists\n          )\n        end\n      end\n    end","ref":"OnePiece.Commanded.TestSupport.CommandHandlerCase.html#module-usage","title":"Usage - OnePiece.Commanded.TestSupport.CommandHandlerCase","type":"module"},{"doc":"","ref":"OnePiece.Commanded.TestSupport.CommandHandlerCase.html#assert_error/5","title":"OnePiece.Commanded.TestSupport.CommandHandlerCase.assert_error/5","type":"function"},{"doc":"","ref":"OnePiece.Commanded.TestSupport.CommandHandlerCase.html#assert_events/5","title":"OnePiece.Commanded.TestSupport.CommandHandlerCase.assert_events/5","type":"function"},{"doc":"","ref":"OnePiece.Commanded.TestSupport.CommandHandlerCase.html#assert_state/5","title":"OnePiece.Commanded.TestSupport.CommandHandlerCase.assert_state/5","type":"function"},{"doc":"# OnePiece.Commanded\n\nExtend `Commanded` package. A swiss army knife for applications following Domain-Driven Design (DDD), Event Sourcing (ES), and Command and Query Responsibility Segregation (CQRS).","ref":"readme.html","title":"OnePiece.Commanded","type":"extras"},{"doc":"#","ref":"readme.html#documentation","title":"Documentation - OnePiece.Commanded","type":"extras"},{"doc":"- [API Reference](api-reference.html)","ref":"readme.html#references","title":"References - OnePiece.Commanded","type":"extras"},{"doc":"# Changelog","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"","ref":"changelog.html#unreleased","title":"Unreleased - Changelog","type":"extras"},{"doc":"- Fixed `new/1` typespec of `OnePiece.Commanded.ValueObject`.","ref":"changelog.html#v0-19-1-2023-10-04","title":"v0.19.1 - 2023-10-04 - Changelog","type":"extras"},{"doc":"`OnePiece.Commanded.ValueObject` implements `Ecto.Type`, it means that `cast/2`, `dump/2`, and `load/2` are\nadded to every module that uses `OnePiece.Commanded.ValueObject`. This will allow you to use \n`OnePiece.Commanded.ValueObject` as a field in your `Ecto.Schema` and as custom type in your\n`OnePiece.Commanded.Aggregate`, `OnePiece.Commanded.Command`, and `OnePiece.Commanded.Event`.\n\nAdded support for custom types for `OnePiece.Commanded.Aggregate`, `OnePiece.Commanded.Command`, and \n`OnePiece.Commanded.Event`. This will allow you to have Custom aggregate identity. Read more about at \nhttps://hexdocs.pm/commanded/commands.html#define-aggregate-identity under \"Custom aggregate identity\".\n\n```elixir\ndefmodule AccountNumber do\n  use OnePiece.Commanded.ValueObject\n  \n  embedded_schema do\n    field :account_number, :string\n    field :branch, :string\n  end\n  \n  # You must implement `String.Chars` protocol in order to work when dispatching the Command. \n  defimpl String.Chars do\n    def to_string(%AccountNumber{branch: branch, account_number: account_number}) do\n      branch   \":\"   account_number\n    end\n  end\nend\n\ndefmodule DepositAccount do\n  use OnePiece.Commanded.Aggregate,\n    identifier: {:account_number, AccountNumber}\n\n  embedded_schema do\n    # ...\n  end\nend\n\ndefmodule DepositAccountOpened do\n  use OnePiece.Commanded.Event,\n    aggregate_identifier: {:account_number, AccountNumber} \n\n  embedded_schema do\n    # ...\n  end\nend\n\ndefmodule OpenDepositAccount do\n  use OnePiece.Commanded.Command,\n    aggregate_identifier: {:account_number, AccountNumber} \n\n  embedded_schema do\n    # ...\n  end\nend\n```","ref":"changelog.html#v0-19-0-2023-06-15","title":"v0.19.0 - 2023-06-15 - Changelog","type":"extras"},{"doc":"- Added `OnePiece.Commanded.ignore_error/2`.","ref":"changelog.html#v0-18-0-2023-05-15","title":"v0.18.0 - 2023-05-15 - Changelog","type":"extras"},{"doc":"- Added `OnePiece.Commanded.Aggregate.StatelessLifespan` module.","ref":"changelog.html#v0-17-0-2023-04-28","title":"v0.17.0 - 2023-04-28 - Changelog","type":"extras"},{"doc":"- Added `OnePiece.Commanded.EventStore.JsonbSerializer` module.","ref":"changelog.html#v0-16-0-2023-04-11","title":"v0.16.0 - 2023-04-11 - Changelog","type":"extras"},{"doc":"- Fix casting already cast structs in `OnePiece.Commanded.ValueObject`.","ref":"changelog.html#v0-15-2-2023-03-14","title":"v0.15.2 - 2023-03-14 - Changelog","type":"extras"},{"doc":"- Fix `OnePiece.Commanded.Event` and `OnePiece.Commanded.Entity` to extend from `OnePiece.Commanded.ValueObject`.","ref":"changelog.html#v0-15-1-2022-12-24","title":"v0.15.1 - 2022-12-24 - Changelog","type":"extras"},{"doc":"- Added overridable to `OnePiece.Commanded.ValueObject.new/1`\n- Added overridable to `OnePiece.Commanded.ValueObject.new!/1`\n- Added overridable to `OnePiece.Commanded.ValueObject.changeset/2`","ref":"changelog.html#v0-15-0-2022-12-23","title":"v0.15.0 - 2022-12-23 - Changelog","type":"extras"},{"doc":"- Added `OnePiece.Commanded.increase_failure_counter/1`","ref":"changelog.html#v0-14-0-2022-09-4","title":"v0.14.0 - 2022-09-4 - Changelog","type":"extras"},{"doc":"- Added `OnePiece.Commanded.tracing_from_metadata/1`.\n- Added `OnePiece.Commanded.tracing_from_metadata/2`.","ref":"changelog.html#v0-13-0-2022-08-19","title":"v0.13.0 - 2022-08-19 - Changelog","type":"extras"},{"doc":"- Added `OnePiece.Commanded.skip_or_retry/2`.\n- Added `OnePiece.Commanded.skip_or_retry/3`.\n- Added `OnePiece.Commanded.Helpers.skip_or_retry/3`.\n- Added `OnePiece.Commanded.Helpers.skip_or_retry/3`.\n- Undo deprecation of `OnePiece.Commanded.cast_to/3`.","ref":"changelog.html#v0-12-0-2022-08-19","title":"v0.12.0 - 2022-08-19 - Changelog","type":"extras"},{"doc":"- Fix register type with a struct with enforce keys in  `OnePiece.Commanded.TypeProvider` module.","ref":"changelog.html#v0-11-1-2022-07-21","title":"v0.11.1 - 2022-07-21 - Changelog","type":"extras"},{"doc":"- Add `OnePiece.Commanded.Helpers.tracing_from_metadata/1`\n- Add `OnePiece.Commanded.Helpers.tracing_from_metadata/2`","ref":"changelog.html#v0-11-0-2022-07-1","title":"v0.11.0 - 2022-07-1 - Changelog","type":"extras"},{"doc":"- Deprecate `OnePiece.Commanded.cast_to/3` (wrong module) and delegate to\n  `OnePiece.Commanded.Helpers` instead.\n- Add `OnePiece.Commanded.Helpers.cast_to/3`","ref":"changelog.html#v0-10-1-2022-06-28","title":"v0.10.1 - 2022-06-28 - Changelog","type":"extras"},{"doc":"- Add `OnePiece.Commanded.cast_to/3`","ref":"changelog.html#v0-10-0-2022-06-23","title":"v0.10.0 - 2022-06-23 - Changelog","type":"extras"},{"doc":"- Move test support under `lib` directory.","ref":"changelog.html#v0-9-1-2022-06-22","title":"v0.9.1 - 2022-06-22 - Changelog","type":"extras"},{"doc":"- Added `OnePiece.Commanded.Id`\n- Deprecated `OnePiece.Commanded.Helpers.generate_uuid/0`. Use\n  `OnePiece.Commanded.Id.new/0` instead.","ref":"changelog.html#v0-7-0-2022-02-14","title":"v0.7.0 - 2022-02-14 - Changelog","type":"extras"},{"doc":"- Added `prefix` option to using `OnePiece.Commanded.TypeProvider`\n\n  ```elixir\n  defmodule MyTypeProvider do\n    use OnePiece.Commanded.TypeProvider, prefix: \"accounts.\"\n  end\n  ```","ref":"changelog.html#v0-6-0-2022-02-14","title":"v0.6.0 - 2022-02-14 - Changelog","type":"extras"},{"doc":"- Added `OnePiece.Commanded.TypeProvider`","ref":"changelog.html#v0-5-0-2022-02-14","title":"v0.5.0 - 2022-02-14 - Changelog","type":"extras"},{"doc":"- Improve documentation","ref":"changelog.html#v0-4-0-2021-11-25","title":"v0.4.0 - 2021-11-25 - Changelog","type":"extras"},{"doc":"- Add `OnePiece.Commanded.Event` module","ref":"changelog.html#v0-4-0-2021-08-15","title":"v0.4.0 - 2021-08-15 - Changelog","type":"extras"},{"doc":"- Fix docs\n- Fix version bump","ref":"changelog.html#v0-3-0-2021-08-15","title":"v0.3.0 - 2021-08-15 - Changelog","type":"extras"},{"doc":"- Add test support files","ref":"changelog.html#v0-2-1-2021-08-15","title":"v0.2.1 - 2021-08-15 - Changelog","type":"extras"},{"doc":"- Fix putting aggregate id to command function name","ref":"changelog.html#v0-2-0-2021-08-08","title":"v0.2.0 - 2021-08-08 - Changelog","type":"extras"},{"doc":"- Fix aliases","ref":"changelog.html#v0-1-3-2021-08-08","title":"v0.1.3 - 2021-08-08 - Changelog","type":"extras"},{"doc":"- Fix `__using__` for aggregate, command handler, query handler, and value\n  object","ref":"changelog.html#v0-1-2-2021-08-08","title":"v0.1.2 - 2021-08-08 - Changelog","type":"extras"},{"doc":"- Remove dead-code","ref":"changelog.html#v0-1-1-2021-08-08","title":"v0.1.1 - 2021-08-08 - Changelog","type":"extras"},{"doc":"- Initial release","ref":"changelog.html#v0-1-0-2021-08-08","title":"v0.1.0 - 2021-08-08 - Changelog","type":"extras"}]}